@@ -1,3 +1,6 @@
+#if !defined(DERIV_H)
+#define DERIV_H
+
 // File deriv.c
 void diffPDF_mod(double* u, double* v, int* n, double* param, int* copula, double* out);
 void diffPDF(double* u, double* v, int* n, double* param, int* copula, double* out);
@@ -11,3 +14,4 @@ void diffhfunc_v_mod(double* u, double* v, int* n, double* param, int* copula, d
 void diffhfunc_v_mod2(double* v, double* u, int* n, double* param, int* copula, double* out);
 void diffhfunc_v(double* u, double* v, int* n, double* param, int* copula, double* out);
 
+#endif
